/* 
*
*  ProLinga-Run
*
*  Copyright (C) 2002-2008 Xobas Software.
*  All rights reserved.
*  
*  This file is part of ProLinga-Run.
*
*  ProLinga-Run is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  ProLinga-Run is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with ProLinga-Run.  If not, see <http://www.gnu.org/licenses/>.
*
*  More information is available at the following addresses:
*
*  Website     : http://www.prolinga.org
*
*  Email       : prolinga-list@prolinga.org
*
*
*/

#include "RunCommon.h"
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <gtk/gtk.h>
#include <pthread.h>
#include <prolinga/Soap.hpp>
#include "RunConfig.hpp"
#include "BuiltIn.hpp"
#include "DataRef.hpp"
#include "CmdList.hpp"
#include "LogicCommandDef.hpp"
#include "RunShare.hpp"
#include <prolinga/Run.hpp>

/* Global Pointers */
PlRunPtr runPtr;

pthread_t mainThreadId = pthread_self();


void ResetRunService(int signo)
{
	/* Close Run */
	runPtr->runClose();

	/* Open Run */
	runPtr->runOpen(NULL, -1, NULL);
}
	
void StopRunService(int signo)
{
	/* Close Data */
	runPtr->runClose();

	/* Exit */
	exit(0);
}

int main(int argc, char **argv)
{
	pid_t pid;
	char *serverHost, *logFile;
	int serverPort, numThreads = 0, compLevel = 0, logLevel = 0, i;
	PlSoapService sservice;
	ConfigRunPtr confRunPtr;

	/* Set global pointer for signal handlers */
	runPtr = new PlRun;

	/* Process command line arguments */
	if (argc > 1)
	{
		for (i = 1; i<argc; i++)
		{
			if (argv[i][0] != '-')
				continue;

			if ((!strcmp(argv[i], "-version")) || (!strcmp(argv[i], "--version")))
			{
				fprintf(stderr,"%s version: %s\n", argv[0], VERSION);
				fprintf(stderr,"Compiled with the following components:\n");
				fprintf(stderr,"     -GNOME/GTK+ GUI\n");
				fprintf(stderr,"     -4GL Interpreter\n");
#ifndef NO_BUILD_SOAP
//				fprintf(stderr,"     -SOAP\n");
#endif
#ifdef BUILD_REPOSITORY
				fprintf(stderr,"     -Repository\n");
#endif
#ifdef BUILD_VALIDATE
				fprintf(stderr,"     -Validate-engine\n");
#endif
#ifdef BUILD_DATA
				fprintf(stderr,"     -Data-engine\n");
#endif
				fprintf(stderr,"Copyright © 2002-2008 The ProLinga Team.\n");
				fprintf(stderr,"All rights reserved.\n");
				return 0;
                        }
                }
        }

	/* Set signal handlers */
	signal(SIGHUP, ResetRunService);
	signal(SIGTERM, StopRunService);

	/* Get Configuration Settings */
	confRunPtr = new ConfigRun(DEF_CONFIG_FILE);

	/* Write configuration */
	logFile = confRunPtr->getStringValue("LogFile", DEF_LOG_FILE);
	confRunPtr->writeConfig(logFile);

	/* Fork Process */
	pid = fork();
	if (pid < 0)
	{
		printf("Error creating server process\n");
		exit(-1);
	}
	else if (pid != 0)
	{
		/* Parent Process */
		exit(0);
	}
	else
	{
		/* Child Process */
		setsid();
		umask(0);
		chdir("/");

		/* Get Service parameters */
		serverHost = confRunPtr->getStringValue("ServerHost", DEF_SERVER_HOST);
		serverPort = confRunPtr->getNumberValue("ServerPort", DEF_SERVER_PORT);
		numThreads = confRunPtr->getNumberValue("NumberOfThreads", DEF_NUMBER_THREADS);
		compLevel = confRunPtr->getNumberValue("ServerCompressionLevel", DEF_SERVER_COMP_LEVEL);
		logLevel = confRunPtr->getNumberValue("LogLevel", DEF_LOG_LEVEL);

		/* Open Run */
		runPtr->runOpen(NULL, argc, argv);

		/*Start the Repository Service */
		sservice.soapServiceStart(serverHost, serverPort, numThreads, compLevel, logFile, logLevel);

		/* Close Run */
		runPtr->runClose();

		/* Clean up */
		delete confRunPtr;

		/* Return */
		return 0;
	}
}

